#!/usr/bin/env python
#-*- coding: utf-8 -*-

## xxgammma - a PyGtk-based GUI for xgamma
##                 <http://www.florian-diesch.de/software/xxgamma/>
## Copyright (C) 2008, 2009  Florian Diesch <devel@florian-diesch.de>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License along
## with this program; if not, write to the Free Software Foundation, Inc.,
## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


APP_NAME='xxgamma'
APP_VERSION='0.06'

import subprocess, os, os.path, sys
from optparse import OptionParser

try:
    import gtk.glade as glade
    import gtk, gobject
except ImportError:
    print '%s needs PyGtk'%APP_NAME
    sys.exit(1)
    
try:
    import shipyard
except ImportError:
    print "%s needs Shipyard <http://www.florian-diesch.de/software/shipyard/>"%APP_NAME
    sys.exit(1)
    
try:
    import peggy
except ImportError:
    print "%s needs Peggy <http://www.florian-diesch.de/software/peggy/>"%APP_NAME
    sys.exit(1)
    

    
CFG_FILE=os.path.join(os.path.expanduser('~/.xxgamma'), 'profiles.sy')


def exec_xgamma(rgb):
    if rgb['single']:
        cmd = ['xgamma', '-gamma', '%(red)s'%rgb]
    else:
        cmd = ['xgamma', '-rgamma', '%(red)s'%rgb,
                         '-ggamma', '%(green)s'%rgb,
                         '-bgamma', '%(blue)s'%rgb]

    try:
        subprocess.check_call(cmd)
    except  subprocess.CalledProcessError, e:
        print e


def get_profiles():
    try:
        inpt=open(CFG_FILE)
        reader=shipyard.Parser(keys=('name', 'red', 'green', 'blue', 'single'))
        result=reader.get_dict(inpt, key='name')
        for r in result.values():
            if r['single'] in ['None', 'False']:
                r['single'] = False
            else:
                r['single'] = bool(r['single'])
        inpt.close()
    except IOError, e:
        #print e
        result={'Default': create_default_profile()}
    return result


def create_default_profile(name=None):
    if name is None:
        name = 'Default'
    return {'name': name, 'single': False,
            'red': 1, 'green': 1, 'blue': 1,
            }

 
class App(object):

    def __init__(self, profile):        
        self.peggy = peggy.Peggy('xxgamma')
        self.peggy.setup_locales()
        self.gladefile = self.peggy.get_glade('data/xxgamma.glade')
        self.gladefile.signal_autoconnect(self)
        self.main_win = self.gladefile.get_widget('main_win')
        self.red = self.gladefile.get_widget('hs_red')
        self.green = self.gladefile.get_widget('hs_green')
        self.blue = self.gladefile.get_widget('hs_blue')
        self.red_label = self.gladefile.get_widget('l_red')
        self.green_label = self.gladefile.get_widget('l_green')
        self.blue_label = self.gladefile.get_widget('l_blue')

        self.single_cb = self.gladefile.get_widget('cb_lock')
        self.image= self.gladefile.get_widget('image')
        self.image.set_from_pixbuf(self.peggy.get_pixbuf('data/cols.xpm'))
        
        self.profiles_box = self.gladefile.get_widget('cbox_profiles')
        self.profiles_store=gtk.ListStore(gobject.TYPE_STRING)
        cell = gtk.CellRendererText()
        self.profiles_box.pack_start(cell, True)
        self.profiles_box.add_attribute(cell, 'text', 0)

        self.dirty = False
        self.is_new = False
        self.single_mode = False
        self.saved_profile = {}
        
        self.read_profiles()
        if profile in self.profiles:
            self.select_profile(profile)
        else:
            try:
                self.select_profile(sorted(self.profiles.keys())[0])
            except IndexError:
                pass



    def ask(self, title, msg):
        dlg = gtk.Dialog(title,
                         self.main_win,
                         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                         (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                          gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        hbox = gtk.HBox()
        label=gtk.Label(msg)
        hbox.pack_start(label, expand=False)
        entry = gtk.Entry()
        entry.set_activates_default(True)
        label.set_use_underline(True)
        label.set_mnemonic_widget(entry)

        hbox.pack_end(entry)
        dlg.vbox.pack_start(hbox)
        dlg.set_default_response(gtk.RESPONSE_ACCEPT)
        dlg.show_all()
        result=None
        if dlg.run()==gtk.RESPONSE_ACCEPT:
            result = entry.get_text()        
        dlg.destroy()
        return result
        
    def run(self):
        try:
            gtk.main()
        except KeyboardInterrupt:
            pass

    def get_rgb_values(self):
        """ get dioct with RGB values from sliders"""
        result={}
        for i in ('red', 'green', 'blue'):
            result[i]=getattr(self, i).get_value()
        result['single'] = self.single_cb.get_active()
        return result

    def set_rgb_values(self, rgb):
        """ set slider values fro m dict"""
        self.single_cb.set_active(bool(rgb['single']))        
        for i in ('red', 'green', 'blue'):
            getattr(self, i).set_value(float(rgb[i]))
        self.dirty=False

        
    def read_profiles(self):
        self.profiles=get_profiles()
        self.update_profiles()

    def set_profile(self, name):
        try:
            rgb=self.profiles[name]
        except KeyError: 
            return
        self.old_profile = rgb
        self.check_dirty()
        self.set_rgb_values(rgb)

    def update_profiles(self, activate=None):
        self.dirty = False
        self.is_new = False
        self.profiles_store.clear()
        for i, p in enumerate(sorted(self.profiles)):
            self.profiles_store.append([p, ])
        self.profiles_box.set_model(self.profiles_store)
        self.select_profile(activate)


    def select_profile(self, name):
        if name is None: return
        active=0
        self.profiles_box.set_active(-1)
        for i, p in enumerate(sorted(self.profiles)):
            if name == p:
                self.profiles_box.set_active(i)
                break

    def write_profiles(self):
        try:
            os.makedirs(os.path.dirname(CFG_FILE))
        except OSError, e:
            print e
        try:
            writer = shipyard.Writer(coding='UTF-8', keys=('name',
                                                           'single',
                                                           'red',
                                                           'green', 'blue'))
            writer.write_full(CFG_FILE, self.profiles.values())
        except IOError, e:
            print e
            
    def save_profile(self, name=None):
        if name is None:
            if self.is_new:
                name = self.ask(title='Save as', msg='Name:')
                if name is None:
                    return
            else:
                name = self.get_current_profile_name()
        self.profiles[name] = self.get_rgb_values()
        self.profiles[name]['name'] =  name
        self.write_profiles()
        self.read_profiles()
        self.select_profile(name)
            

    def get_current_profile_name(self):
        return self.profiles_box.get_active_text()
        
        
    def check_dirty(self):
        if self.dirty:
            msg = gtk.MessageDialog(self.main_win,
                                  gtk.DIALOG_MODAL
                                  | gtk.DIALOG_DESTROY_WITH_PARENT,
                                  gtk.MESSAGE_WARNING,
                                  gtk.BUTTONS_YES_NO,
                                  'The current gamma values are not saved. '
                                  'Do you want to save them now?')
            msg.set_default_response(gtk.RESPONSE_YES)
            answer = msg.run()
            msg.destroy()
            if answer == gtk.RESPONSE_YES:
                self.save_profile()
            self.dirty = False
            self.is_new = False
            self.read_profiles()
            
    def quit(self):
        self.check_dirty()
        gtk.main_quit()

        
    def on_mi_quit_activate(self, *args):
        self.quit()

    def on_hs_value_changed(self, widget, *args):
        self.dirty=True
        rgb=self.get_rgb_values()
        exec_xgamma(rgb)

    def on_mi_new_activate(self, *args):
        self.check_dirty()
        i=0
        name='--Unnamed_%s--'
        while name%i in self.profiles:
            i+=1
        self.profiles_store.append([name%i, ])
        self.profiles_box.set_active(len(self.profiles_store)-1)
        self.is_new=True
        self.dirty=True

    def on_mi_save_activate(self, *args):
        self.save_profile()
        
    def on_mi_saveas_activate(self, *args):
        name = self.ask(title='Save as', msg='Name:')
        if name is not None:
            self.save_profile(name=name)

    def on_mi_revert_activate(self, *args):
        current=self.get_current_profile_name()
        self.profiles[current] = self.old_profile
        self.set_rgb_values(self.old_profile)
        self.dirty = False
        

 
    def on_mi_delete_activate(self, *args):
       current=self.get_current_profile_name()
       if current in self.profiles:
           del self.profiles[current]
       self.write_profiles()
       self.read_profiles()
       try:
           self.select_profile(self.profiles.keys()[0])
       except IndexError:
           pass


    def on_mi_info_activate(self, *args):
        dlg = gtk.AboutDialog()
        dlg.set_name(APP_NAME)
        dlg.set_version(APP_VERSION)
        dlg.set_website('http://www.florian-diesch.de/software/xxgamma/')
        dlg.set_authors(['Florian Diesch <devel@florian-diesch.de>'])
        dlg.set_copyright('Copyright (C) 2008  Florian Diesch ')
        dlg.set_license(
            "xxgammma - a PyGtk-based GUI for xgamma\n"
            "                <http://www.florian-diesch.de/software/xxgamma/>\n"
            "Copyright (C) 2008  Florian Diesch <devel@florian-diesch.de>\n"
            "\n"
            "This program is free software; you can redistribute it and/or"
            "modify\n"
            "it under the terms of the GNU General Public License as published"
            "by\n"
            "the Free Software Foundation; either version 2 of the License, or"
            "(at your option) any later version.\n"
            "\n"
            "This program is distributed in the hope that it will be useful,\n"
            "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
            "GNU General Public License for more details.\n"
            "\n"
            "You should have received a copy of the GNU General Public License"
            "along\n"
            "with this program; if not, write to the Free Software Foundation,"
            "Inc.,\n"
            "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n")
        
        dlg.run()
        dlg.destroy()

    def on_cb_lock_toggled(self, widget, *args):
        self.single_mode = widget.get_active()

        for w in (self.green, self.blue, self.green_label, self.blue_label):
            w.set_sensitive(not self.single_mode)
        self.red_label.set_label(['_Red', '_Gamma'][self.single_mode])
        rgb = self.get_rgb_values()
        if not self.single_mode:
            rgb['green'] = rgb['blue'] = rgb['red']
            self.set_rgb_values(rgb)

    def on_cbox_profiles_changed(self, widget, *args):
        name = self.get_current_profile_name()
        self.set_profile(name)

    def on_mainwin_delete_event(self, *args):
        self.quit()


def main():
    parser = OptionParser(version="%s %s "%(APP_NAME, APP_VERSION))
    parser.add_option("-n", "--no-gui", dest="nogui", action="store_true",
                      help="don't show the GUI, just set the gamma profile")
    parser.add_option("-p", "--profile", dest="profile", default='Default',
                      help="load profile named PROFILE", metavar="PROFILE")

    parser.add_option("-l", "--list", dest="list", action="store_true",
                      help="list existing profiles and exit")

    (options, args) = parser.parse_args()
    
    if options.list:
        print '\n'.join(get_profiles().keys())
    elif options.nogui:
        profiles=get_profiles()
        if options.profile in profiles:
            exec_xgamma(profiles[options.profile])
        else:
            print 'Profile %s not found.'%options.profile
    else:
        app=App(options.profile)
        app.run()

if __name__ == '__main__':
    main()

